@@ -31,6 +31,7 @@
 #include <QResizeEvent>
 #include <QPalette>
 #include <QTimer>
+#include <QScrollBar>
 
 #include <kdebug.h>
 #include <kdialog.h>
@@ -79,7 +80,12 @@
 #include "importitemdialog.h"
 #include "addressprovider.h"
 #include "doclocaledialog.h"
+#include "akonadiaddressselectordialog.h"
 
+#define NO_TAX   0
+#define RED_TAX  1
+#define FULL_TAX 2
+#define INDI_TAX 3
 
 KraftViewScroll::KraftViewScroll( QWidget *parent ):
 QScrollArea( parent )
@@ -88,17 +94,26 @@ QScrollArea( parent )
   myWidget->setAutoFillBackground(false);
   layout = new QVBoxLayout;
   layout->setAlignment(Qt::AlignTop);
-  layout->setSizeConstraint( QLayout::QLayout::SetMinAndMaxSize );
-  layout->setMargin( 0 );
+  layout->setSizeConstraint( QLayout::SetMinAndMaxSize );
+  layout->setContentsMargins( 0,0,0,0 );
+  layout->setSpacing(0);
   myWidget->setLayout(layout);
   setWidget(myWidget);
   setWidgetResizable(true);
-
+  myWidget->resize(0,0);
+  myWidget->setMinimumHeight(0);
+  myWidget->setMaximumHeight(0);
+  myWidget->setContentsMargins(0, 0, 0, 0);
 }
 
 void KraftViewScroll::addChild( QWidget *child, int index )
 {
-  layout->insertWidget(index, child);
+    int y1 = myWidget->height();
+    layout->insertWidget(index, child);
+    int y2 = y1+child->height();
+    myWidget->resize( child->width(), y2);
+    myWidget->setMinimumHeight(y2);
+    myWidget->setMaximumHeight(y2);
 }
 
 void KraftViewScroll::removeChild( PositionViewWidget *child )
@@ -300,6 +315,7 @@ void KraftView::setupDocHeaderView()
              this, SLOT( slotLanguageSettings() ) );
     connect( edit, SIGNAL( modified() ),
               this, SLOT( slotModifiedHeader() ) );
+    connect( edit, SIGNAL(pickAddressee()), this, SLOT(slotPickAddressee()) );
 }
 
 void KraftView::setupPositions()
@@ -367,6 +383,15 @@ void KraftView::redrawDocument( )
     mModified = false;
 }
 
+void KraftView::slotPickAddressee()
+{
+    AkonadiAddressSelectorDialog dialog(this);
+
+    if( dialog.exec() ) {
+        slotNewAddress( dialog.addressee() );
+    }
+}
+
 void KraftView::slotAddresseeFound( const QString& uid, const KABC::Addressee& contact )
 {
   kDebug() << "Addressee Found with uid " << uid;
@@ -384,7 +409,7 @@ void KraftView::redrawDocPositions( )
   if ( list.count() == 0 ) {
     // the doc has no positions yet. Let's show a help page
     if ( ! mHelpLabel ) {
-      mHelpLabel = new QLabel(0);
+      mHelpLabel = new QLabel(this);
       mHelpLabel->setMargin( KDialog::marginHint() );
       mHelpLabel->setText( i18n( "<qt><h2>The Document Items List is still empty, but Items "
                                  "can be added now.</h2>"
@@ -414,6 +439,7 @@ void KraftView::redrawDocPositions( )
     PositionViewWidget *w = mPositionWidgetList.widgetFromPosition( dp );
     if( !w ) {
       w = createPositionViewWidget( dp, cnt);
+      w->slotAllowIndividualTax( currentTaxSetting() == DocPositionBase::TaxIndividual );
     }
     cnt++;
     kDebug() << "now position " << dp->positionNumber() << endl;
@@ -499,7 +525,7 @@ PositionViewWidget *KraftView::createPositionViewWidget( DocPositionBase *dp, in
 
   w->setDocPosition( dp, getDocument()->locale() );
   w->setOrdNumber( pos+1 );
-
+  w->slotSetTax( dp->taxType() );
   return w;
 }
 
@@ -515,8 +541,8 @@ DocPositionBase::TaxType KraftView::currentTaxSetting()
     tt = DocPositionBase::TaxReduced;
   } else if ( taxKind == 3 ) { // Full tax for all items
     tt = DocPositionBase::TaxFull;
-  } else { // individual level, not yet implementend
-    kError() << "Item individual tax level is not yet implemented.";
+  } else { // individual level
+    tt = DocPositionBase::TaxIndividual;
   }
   return tt;
 }
@@ -577,9 +603,11 @@ void KraftView::setupFooter()
 
   // ATTENTION: If you change the following inserts, make sure to check the code
   //            in method currentPositionList!
-  m_footerEdit->mTaxCombo->insertItem( 0, i18n( "Display no tax at all" ));
-  m_footerEdit->mTaxCombo->insertItem( 1, i18n( "Calculate reduced tax for all items" ));
-  m_footerEdit->mTaxCombo->insertItem( 2, i18n( "Calculate full tax for all items" ));
+  m_footerEdit->mTaxCombo->insertItem( NO_TAX,   KIcon("kraft_notax"), i18n( "Display no tax at all" ));
+  m_footerEdit->mTaxCombo->insertItem( RED_TAX,  KIcon("kraft_redtax"), i18n( "Calculate reduced tax for all items" ));
+  m_footerEdit->mTaxCombo->insertItem( FULL_TAX, KIcon("kraft_fulltax"), i18n( "Calculate full tax for all items" ));
+  m_footerEdit->mTaxCombo->insertItem( INDI_TAX, i18n( "Calculate individual tax for each item"));
+
   // m_footerEdit->mTaxCombo->insertItem( i18n( 3, i18n( "Calculate on individual item tax rate" )));
 
   // set the tax type combo correctly: If all items have the same tax type, take it.
@@ -588,17 +616,17 @@ void KraftView::setupFooter()
 
   int tt = -1;
   DocPositionBase *dp = 0;
-  bool equality = true;
 
   DocPositionListIterator it( list );
+  int taxIndex = 0;
+
   while( it.hasNext()) {
     dp = it.next();
     if ( tt == -1 )
       tt = dp->taxTypeNumeric(); // store the first entry.
     else {
       if ( tt != dp->taxTypeNumeric() ) {
-        m_footerEdit->mTaxCombo->setCurrentIndex( 3 );
-        equality = false;
+        taxIndex = INDI_TAX;
       } else {
         // old and new taxtype are the same.
       }
@@ -610,20 +638,21 @@ void KraftView::setupFooter()
     if ( deflt > 0 ) {
       deflt -= 1;
     }
-    m_footerEdit->mTaxCombo->setCurrentIndex( deflt );
+    taxIndex = deflt;
   } else {
-    if ( equality ) {
-      m_footerEdit->mTaxCombo->setCurrentIndex( tt-1 );
-    } else {
-      kError() << "Problem: Not all Tax-Levels are the same! (Fixed later with tax on item base)";
+    if ( taxIndex == 0 ) {
+      taxIndex = tt-1;
     }
   }
 
-  connect( m_footerEdit->mTaxCombo, SIGNAL( activated( int ) ),
-           this, SLOT( slotModifiedFooter() ) );
+  connect( m_footerEdit->mTaxCombo,SIGNAL(currentIndexChanged(int)),this,SLOT(slotTaxComboChanged(int)));
 
-  connect( edit, SIGNAL( modified() ),
-           this, SLOT( slotModifiedFooter() ) );
+  m_footerEdit->mTaxCombo->setCurrentIndex( taxIndex );
+  slotTaxComboChanged( taxIndex );
+
+  mTaxBefore = taxIndex;
+
+  connect(edit,SIGNAL(modified()),this,SLOT(slotModifiedFooter()));
 }
 
 void KraftView::slotAboutToShow( QWidget* w )
@@ -631,6 +660,42 @@ void KraftView::slotAboutToShow( QWidget* w )
   kDebug() << "showing page " << w << endl;
 }
 
+void KraftView::slotTaxComboChanged(int newId)
+{
+  bool allowTaxSetting = false;
+
+  if( mTaxBefore == newId ) return;
+
+  if( mTaxBefore == INDI_TAX ) {
+    // we're changing away from individual settings. WARNING needed.
+    kDebug() << "You switch away from item individual tax settings.";
+    if( KMessageBox::questionYesNo( this, i18n("Really overwrite all individual tax settings of the items?"),
+                                         i18n("Tax Settings Overwrite") ) == KMessageBox::No ) {
+      m_footerEdit->mTaxCombo->setCurrentIndex( INDI_TAX );
+      return;
+    }
+  }
+  DocPositionBase::TaxType tax = DocPositionBase::TaxFull;
+
+  if( newId == INDI_TAX ) {
+    allowTaxSetting = true;
+  } else if( newId == RED_TAX ) {
+    tax = DocPositionBase::TaxReduced;
+  } else if( newId == NO_TAX ) {
+    tax = DocPositionBase::TaxNone;
+  }
+
+  PositionViewWidgetListIterator it( mPositionWidgetList );
+  while( it.hasNext() ) {
+    PositionViewWidget *w = it.next();
+    w->slotAllowIndividualTax( allowTaxSetting );
+    w->slotSetTax( tax );
+  }
+
+  mTaxBefore = newId;
+  slotModifiedFooter();
+}
+
 /* This is the flow in the move up method:
          0    Bla1          0 Bla1                         0 Bla1
          1    Bla2 <- w2    1 Bla2   -> insert at pos-1 => 1 Bla3
@@ -766,6 +831,7 @@ void KraftView::slotNewAddress( const Addressee& contact, bool interactive )
   const QString currAddress = m_headerEdit->m_postAddressEdit->toPlainText();
 
   bool replace = true;
+  m_headerEdit->m_labName->setText( contact.realName() );
 
   if( currAddress.isEmpty() ) {
     replace = true;
@@ -773,7 +839,8 @@ void KraftView::slotNewAddress( const Addressee& contact, bool interactive )
     // non empty and current different from new address
     // need to ask first if we overwrite
     if( interactive ) {
-      if( KMessageBox::questionYesNo( this, i18n("The address label is not empty. Do you really want to replace it?"),
+        if( KMessageBox::questionYesNo( this, i18n("The address label is not empty and different from the selected one.<br/>"
+                                                   "Do you really want to replace it with the text shown below?<pre>%1</pre>").arg(newAddress),
                                      i18n("Address Overwrite") ) == KMessageBox::No ) replace = false;
     } else {
       // this happens when the document is loaded and the address arrives from addressbook
@@ -785,7 +852,6 @@ void KraftView::slotNewAddress( const Addressee& contact, bool interactive )
   }
 
   if( replace ) {
-    m_headerEdit->m_labName->setText( contact.realName() );
     mContactUid = contact.uid();
 
     m_headerEdit->m_postAddressEdit->setText( newAddress );
@@ -941,7 +1007,13 @@ void KraftView::slotAddItem( Katalog *kat, CatalogTemplate *tmpl )
       *dp = diaPos;
 
       // set the tax settings
-      dp->setTaxType( currentTaxSetting() );
+      if( currentTaxSetting() == DocPositionBase::TaxIndividual ) {
+        // FIXME: In case a new item is added, add the default tax type.
+        // otherwise add the tax of the template
+        dp->setTaxType( DocPositionBase::TaxFull );
+      } else {
+        dp->setTaxType( currentTaxSetting() );
+      }
 
       // store the initial size of the template-to-doc-pos dialogs
       s = dia->size();
@@ -951,8 +1023,12 @@ void KraftView::slotAddItem( Katalog *kat, CatalogTemplate *tmpl )
 
         // if it's a new position, create a catalog template in the incoming chapter
         if ( newTemplate ) {
-          // const QString chapter = dia->chapter();
-          int chapterId = 0; // FIXME - need a new way of chapter picking
+          const QString chapter = dia->chapter();
+
+          int chapterId = 0; // find the chapter.
+          if( !chapter.isEmpty() ) {
+              chapterId = KatalogMan::self()->defaultTemplateCatalog()->chapterID(chapter).toInt();
+          }
 
           FloskelTemplate *flos = new FloskelTemplate( -1, dp->text(),
                                                        dp->unit().id(),
@@ -960,9 +1036,14 @@ void KraftView::slotAddItem( Katalog *kat, CatalogTemplate *tmpl )
                                                        1 /* CalcKind = Manual */ );
 
           flos->setManualPrice( dp->unitPrice() );
-          flos->save(); // <- Checke das hier!
-          mNewTemplates.append( flos );
+          flos->save();
 
+          // reload the entire katalog
+          Katalog *defaultKat = KatalogMan::self()->defaultTemplateCatalog();
+          if( defaultKat ) {
+              defaultKat->load();
+              KatalogMan::self()->notifyKatalogChange( defaultKat , dbID() );
+          }
         }
       } else if ( kat->type() == MaterialCatalog ) {
         KraftSettings::self()->setTemplateToPosDialogSize( s );
@@ -992,7 +1073,9 @@ void KraftView::slotAddItem( Katalog *kat, CatalogTemplate *tmpl )
 
   PositionViewWidget *widget = createPositionViewWidget( dp, newpos );
   widget->slotModified();
-  slotFocusPosition( widget, newpos );
+  widget->slotAllowIndividualTax( currentTaxSetting() == DocPositionBase::TaxIndividual );
+
+  slotFocusItem( widget, newpos );
   refreshPostCard();
 }
 
@@ -1018,6 +1101,7 @@ void KraftView::slotImportItems()
         DocPosition *dp = new DocPosition( *(dp_old) );
         dp->setTaxType( currentTaxSetting() );
         PositionViewWidget *widget = createPositionViewWidget( dp, newpos + cnt++ );
+        widget->slotSetTax( DocPositionBase::TaxFull ); // FIXME: Value from Import?
         widget->slotModified();
       }
       refreshPostCard();
@@ -1034,14 +1118,18 @@ void KraftView::slotAddExtraPosition()
   DocPosition *dp = new DocPosition( DocPosition::ExtraDiscount );
   dp->setPositionNumber( newpos+1 );
   dp->setText( i18n( "Discount" ) );
-  dp->setTaxType( currentTaxSetting() );
+  if( currentTaxSetting() == DocPositionBase::TaxIndividual ) {
+    dp->setTaxType( DocPositionBase::TaxFull );
+  } else {
+    dp->setTaxType( currentTaxSetting() );
+  }
 
   kDebug() << "New Extra position is " << dp << endl;
 
   PositionViewWidget *widget = createPositionViewWidget( dp, newpos );
   kDebug() << "PositionViewWiget doc position is: " << widget->position() << endl;
   widget->slotModified();
-  slotFocusPosition( widget, newpos );
+  slotFocusItem( widget, newpos );
   refreshPostCard();
 
 }
@@ -1186,7 +1274,11 @@ DocPositionList KraftView::currentPositionList()
             }
 
             // tax settings
-            newDp->setTaxType( currentTaxSetting() );
+            if( currentTaxSetting() == DocPositionBase::TaxIndividual ) {
+              newDp->setTaxType( widget->taxType() );
+            } else {
+              newDp->setTaxType( currentTaxSetting() );
+            }
             list.append( newDp );
           }
         } else {
@@ -1336,33 +1428,17 @@ void KraftView::saveChanges()
     KraftSettings::self()->setDocViewPosition( pos() );
     KraftSettings::self()->writeConfig();
     KraftSettings::self()->readConfig();
-
-    // Save newly created templates
-    if ( mNewTemplates.count() > 0 ) {
-      CatalogTemplate *ct = 0;
-      CatalogTemplateListIterator it( mNewTemplates );
-      while( it.hasNext()) {
-        ct = it.next();
-        ct->save();
-      }
-      mNewTemplates.clear();
-      // reload the entire katalog
-      Katalog *defaultKat = KatalogMan::self()->defaultTemplateCatalog();
-      KatalogMan::self()->notifyKatalogChange( defaultKat , dbID() );
-    }
 }
 
-void KraftView::slotFocusPosition( PositionViewWidget *posWidget, int pos )
+void KraftView::slotFocusItem( PositionViewWidget *posWidget, int pos )
 {
   if( posWidget && pos > 0) {
-    int w = posWidget->height();
-    kDebug() << "Focussing on widget " << posWidget << " on pos " << pos << " with heigt " << w;
-    int y = (pos)*w;
-    kDebug() << "Focus on y-koord " << y << " on canvas height " << (m_positionScroll->widget())->height();
-    m_positionScroll->ensureVisible( 2, y, 0, 0 );
+    int y = (1+pos)*posWidget->height();
+    m_positionScroll->ensureVisible(0, y);
   } else {
     m_positionScroll->ensureVisible( 0, 0 );
   }
+  // setting Focus within the item.
   if( posWidget ) {
     if( posWidget->m_teFloskel->toPlainText().isEmpty() ) {
       posWidget->m_teFloskel->setFocus();
@@ -1394,8 +1470,6 @@ void KraftView::discardChanges()
     kDebug() << "Document refetch from database" << endl;
     doc->reloadDocument();
   }
-
-  mNewTemplates.clear();
 }
 
 void KraftView::print(QPrinter * /* pPrinter */ )
