@@ -345,6 +345,11 @@ void Portal::slotStartupChecks()
 
 void Portal::slotReceivedMyAddress( const QString& uid, const KABC::Addressee& contact )
 {
+  if( contact.isEmpty() ) {
+    kDebug() << "My-Contact is empty!";
+    return;
+  }
+
   myContact = contact;
 
   KraftSettings::self()->setUserUid( contact.uid() );
@@ -379,9 +384,6 @@ void Portal::busyCursor( bool on )
 void Portal::slotNewDocument()
 {
   slotStatusMsg(i18n("Creating new document..."));
-  busyCursor( true );
-
-  busyCursor( false );
 
   KraftWizard wiz;
   wiz.init();
@@ -419,7 +421,7 @@ void Portal::slotFollowUpDocument()
   kDebug() << "doc identifier: "<< doc->docIdentifier() << endl;
   wiz.setDocIdentifier( doc->docIdentifier() );
   if ( wiz.exec() ) {
-    DocGuardedPtr doc = DocumentMan::self()->createDocument( locId );
+    DocGuardedPtr doc = DocumentMan::self()->createDocument( dt.name(), locId );
     doc->setDate( wiz.date() );
     doc->setDocType( wiz.docType() );
     doc->setWhiteboard( wiz.whiteboard() );
@@ -438,11 +440,12 @@ void Portal::slotCopyDocument( const QString& id )
   if ( id.isEmpty() ) {
     return;
   }
+  DocGuardedPtr oldDoc = DocumentMan::self()->openDocument( id );
 
   KraftWizard wiz;
   wiz.init();
   if ( wiz.exec() ) {
-    DocGuardedPtr doc = DocumentMan::self()->createDocument( id );
+    DocGuardedPtr doc = DocumentMan::self()->createDocument( oldDoc->docType(), id );
     doc->setDate( wiz.date() );
     doc->setDocType( wiz.docType() );
     doc->setWhiteboard( wiz.whiteboard() );
